# Generated by Django 4.1.5 on 2023-02-17 13:46
from decimal import Decimal

import django.core.validators
from django.db import migrations

import audoma.django.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("audoma_api", "0006_cartag"),
    ]

    operations = [
        migrations.AlterField(
            model_name="examplemodel",
            name="decimal",
            field=audoma.django.db.fields.DecimalField(
                decimal_places=2,
                max_digits=10,
                validators=[django.core.validators.MinValueValidator(Decimal("1.0"))],
            ),
        ),
    ]
